# frozen_string_literal: true

require 'git'
require 'json'
require 'date'
require '/data_importer/lib/importers/github_repo'
require '/data_importer/lib/json_helper'

# This class can be used to import cvelist json data from mitre from their github repo
class CveListImporter < GithubRepo
  EXPECTED_KEYS = %i[
    cve_data_meta
    cve_id
    affects
    data_format
    data_type
    data_version
    description
    impact
    problemtype
    references
    source
  ].freeze

  EMPTY_HASH = EXPECTED_KEYS.map { |k| [k, nil] }.to_h.freeze

  def initialize
    super(repo_url = 'https://github.com/CVEProject/cvelist.git', repo_path = '/data_importer/data/cve_list')
  end

  def list_jsons_for_year(year)
    year_fp = "#{repo_path}/#{year}/*"
    Dir["#{year_fp}/*.json"]
  end

  def read_jsons_for_year(year)
    filenames = list_jsons_for_year(year)
    hashes = filenames.map do |filename|
      json = read_json(filename)
      json_transformed = JsonHelper.deep_transform_keys(json)
      add_cve_id_to_json_key(json_transformed)
    end
    hashes.map { |h| h.slice(*EXPECTED_KEYS).reverse_merge(EMPTY_HASH) }
  end

  def add_cve_id_to_json_key(json)
    json[:cve_id] = json[:cve_data_meta][:id]
    json
  end

  def import
    pull_or_clone
    puts "Now starting import for #{repo_url}."
    puts '----------' * 12
    (1999..Date.today.year).map do |year|
      cves = read_jsons_for_year(year)
      puts "Importing any new CVEs from #{year}"
      Cve.upsert_all(cves, unique_by: :cve_id)
    end
  end
end
